#!/usr/bin/env python
import os
import sys

from django.core.management.templates import TemplateCommand
from django.core.management.utils import get_random_secret_key


CURRENT_PYTHON = sys.version_info[:2]
REQUIRED_PYTHON = (3, 4)

if CURRENT_PYTHON < REQUIRED_PYTHON:
    sys.stderr.write(
        "This version of Wagtail requires Python {}.{} or above - you are running {}.{}\n".format(*(REQUIRED_PYTHON + CURRENT_PYTHON))  # noqa
    )
    sys.exit(1)


class CreateProject(TemplateCommand):
    """
    Based on django.core.management.startproject
    """
    help = "Creates the directory structure for a new CodeRed CMS project."
    missing_args_message = "You must provide a project name."

    def add_arguments(self, parser):
        parser.add_argument(
            '--sitename',
            help='Human readable name of your website or brand, e.g. "Mega Corp Inc."'
        )
        parser.add_argument(
            '--domain',
            help='Domain that will be used for your website in production, e.g. "www.example.com"'
        )
        super().add_arguments(parser)

    def handle(self, **options):
        # pop standard args
        project_name = options.pop('name')
        target = options.pop('directory')

        # Make sure given name is not already in use by another python package/module.
        try:
            __import__(project_name)
        except ImportError:
            pass
        else:
            sys.exit("'%s' conflicts with the name of an existing "
                     "Python module and cannot be used as a project "
                     "name. Please try another name." % project_name)

        # Create a random SECRET_KEY to put it in the main settings.
        options['secret_key'] = get_random_secret_key()

        # Handle custom template logic
        import coderedcms
        codered_path = os.path.dirname(coderedcms.__file__)
        if not options['template']:
            options['template'] = 'basic'
        template_path = os.path.join(
            os.path.join(codered_path, 'project_template'),
            options['template']
        )

        # Check if provided template is built-in to coderedcms,
        # otherwise, do not change it.
        if os.path.isdir(template_path):
            options['template'] = template_path

        # Treat these files as Django templates to render the boilerplate.
        options['extensions'] = ['py', 'md', 'txt']
        options['files'] = ['Dockerfile']

        # Set options
        message = "Creating a CodeRed CMS project called %(project_name)s"

        if options.get('sitename'):
            message += " for %(sitename)s"
        else:
            options['sitename'] = project_name

        if options.get('domain'):
            message += " (%(domain)s)"
            # Stip protocol out of domain if it is present.
            options['domain'] = options['domain'].split('://')[-1]
            # Figure out www logic.
            if options['domain'].startswith('www.'):
                options['domain_nowww'] = options['domain'].split('www.')[-1]
            else:
                options['domain_nowww'] = options['domain']
        else:
            options['domain'] = 'localhost'
            options['domain_nowww'] = options['domain']

        # Add additional custom options to the context.
        options['coderedcms_release'] = coderedcms.release

        # Print a friendly message
        print(message % {
            'project_name': project_name,
            'sitename': options.get('sitename'),
            'domain': options.get('domain'),
        })

        # Run command
        super().handle('project', project_name, target, **options)

        # Be a friend once again.
        print("Success! %(project_name)s has been created" % {'project_name': project_name})

        nextsteps = """
Next steps:
    1. cd %(directory)s/
    2. python manage.py migrate
    3. python manage.py createsuperuser
    4. python manage.py runserver
    5. Go to http://localhost:8000/admin/ and start editing!
"""
        print(nextsteps % {'directory': target if target else project_name})


COMMANDS = {
    'start': CreateProject(),
}


def prog_name():
    return os.path.basename(sys.argv[0])


def help_index():
    print("Type '%s help <subcommand>' for help on a specific subcommand.\n" % prog_name())  # NOQA
    print("Available subcommands:\n")  # NOQA
    for name, cmd in sorted(COMMANDS.items()):
        print("    %s%s" % (name.ljust(20), cmd.help))  # NOQA


def unknown_command(command):
    print("Unknown command: '%s'" % command)  # NOQA
    print("Type '%s help' for usage." % prog_name())  # NOQA
    sys.exit(1)


def main():
    try:
        command_name = sys.argv[1]
    except IndexError:
        help_index()
        return

    if command_name == 'help':
        try:
            help_command_name = sys.argv[2]
        except IndexError:
            help_index()
            return

        try:
            command = COMMANDS[help_command_name]
        except KeyError:
            unknown_command(help_command_name)
            return

        command.print_help(prog_name(), help_command_name)
        return

    try:
        command = COMMANDS[command_name]
    except KeyError:
        unknown_command(command_name)
        return

    command.run_from_argv(sys.argv)


if __name__ == "__main__":
    main()



from django.urls import include, path, re_path
from wagtailimportexport import urls as wagtailimportexport_urls
from wagtail.admin import urls as wagtailadmin_urls
from coderedcms.views import import_pages_from_csv_file


urlpatterns = [
    path('codered/import-export/import_from_csv/',
         import_pages_from_csv_file, name="import_from_csv"),
    re_path(r'', include(wagtailadmin_urls)),
    re_path(r'', include(wagtailimportexport_urls)),
]



from django.apps import AppConfig


class CoderedcmsConfig(AppConfig):
    name = 'coderedcms'
    verbose_name = 'CodeRed CMS'
    # TODO: At some point in the future, change this to BigAutoField and create
    # the corresponding migration for concrete models in coderedcms.
    default_auto_field = 'django.db.models.AutoField'



from django.template.loader import render_to_string
from django.utils.safestring import mark_safe
from django.utils.translation import gettext as _

from wagtail.admin.edit_handlers import EditHandler


class FormSubmissionsPanel(EditHandler):
    template = "wagtailforms/edit_handlers/form_responses_panel.html"

    def bind_to(self, model=None, instance=None, request=None, form=None):
        new = super().bind_to(model=model)
        if self.heading is None:
            new.heading = _('{} submissions').format(model.get_verbose_name())
        return new

    def render(self):
        Submission = self.model.get_submission_class()
        submissions = Submission.objects.filter(page=self.instance)
        submission_count = submissions.count()

        if not submission_count:
            return ''

        return mark_safe(render_to_string(self.template, {
            'self': self,
            'submission_count': submission_count,
            'last_submit_time': (submissions.order_by('submit_time')
                                 .last().submit_time),
        }))



from django.utils.translation import gettext_lazy as _
from wagtail.core import blocks

from coderedcms.wagtail_flexible_forms import blocks as form_blocks
from coderedcms.blocks.base_blocks import BaseBlock, CoderedAdvSettings
from coderedcms.forms import (
    CoderedDateField, CoderedDateInput,
    CoderedDateTimeField, CoderedDateTimeInput,
    CoderedTimeField, CoderedTimeInput,
    SecureFileField
)


class CoderedFormAdvSettings(CoderedAdvSettings):

    condition_trigger_id = blocks.CharBlock(
        required=False,
        max_length=255,
        label=_('Condition Trigger ID'),
        help_text=_(
            'The "Custom ID" of another field that that will trigger this field to be shown/hidden.')  # noqa
    )
    condition_trigger_value = blocks.CharBlock(
        required=False,
        max_length=255,
        label=_('Condition Trigger Value'),
        help_text=_(
            'The value of the field in "Condition Trigger ID" that will trigger this field to be shown.')  # noqa
    )


class FormBlockMixin(BaseBlock):
    class Meta:
        abstract = True

    advsettings_class = CoderedFormAdvSettings


class CoderedStreamFormFieldBlock(form_blocks.OptionalFormFieldBlock, FormBlockMixin):
    pass


class CoderedStreamFormCharFieldBlock(form_blocks.CharFieldBlock, FormBlockMixin):
    class Meta:
        label = _("Text or Email input")
        icon = "fa-window-minimize"


class CoderedStreamFormTextFieldBlock(form_blocks.TextFieldBlock, FormBlockMixin):
    class Meta:
        label = _("Multi-line text")
        icon = "fa-align-left"


class CoderedStreamFormNumberFieldBlock(form_blocks.NumberFieldBlock, FormBlockMixin):
    class Meta:
        label = _("Numbers only")
        icon = "fa-hashtag"


class CoderedStreamFormCheckboxFieldBlock(form_blocks.CheckboxFieldBlock, FormBlockMixin):
    class Meta:
        label = _("Single Checkbox")
        icon = "fa-check-square-o"


class CoderedStreamFormRadioButtonsFieldBlock(form_blocks.RadioButtonsFieldBlock, FormBlockMixin):
    class Meta:
        label = _("Radios")
        icon = "fa-list-ul"


class CoderedStreamFormDropdownFieldBlock(form_blocks.DropdownFieldBlock, FormBlockMixin):
    class Meta:
        label = _("Dropdown")
        icon = "fa-list-alt"


class CoderedStreamFormCheckboxesFieldBlock(form_blocks.CheckboxesFieldBlock, FormBlockMixin):
    class Meta:
        label = _("Checkboxes")
        icon = "fa-list-ul"


class CoderedStreamFormDateFieldBlock(form_blocks.DateFieldBlock, FormBlockMixin):
    class Meta:
        label = _("Date")
        icon = "fa-calendar"

    field_class = CoderedDateField
    widget = CoderedDateInput


class CoderedStreamFormTimeFieldBlock(form_blocks.TimeFieldBlock, FormBlockMixin):
    class Meta:
        label = _("Time")
        icon = "fa-clock-o"

    field_class = CoderedTimeField
    widget = CoderedTimeInput


class CoderedStreamFormDateTimeFieldBlock(form_blocks.DateTimeFieldBlock, FormBlockMixin):
    class Meta:
        label = _("Date and Time")
        icon = "fa-calendar"

    field_class = CoderedDateTimeField
    widget = CoderedDateTimeInput


class CoderedStreamFormImageFieldBlock(form_blocks.ImageFieldBlock, FormBlockMixin):
    class Meta:
        label = _("Image Upload")
        icon = "fa-picture-o"


class CoderedStreamFormFileFieldBlock(form_blocks.FileFieldBlock, FormBlockMixin):
    class Meta:
        label = _("Secure File Upload")
        icon = "fa-upload"

    field_class = SecureFileField


class CoderedStreamFormStepBlock(form_blocks.FormStepBlock):
    form_fields = blocks.StreamBlock()

    def __init__(self, local_blocks=None, **kwargs):
        super().__init__(
            local_blocks=[
                ('form_fields', blocks.StreamBlock(local_blocks))
            ]
        )



# Generated by Django 2.1.5 on 2019-01-14 20:15

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('coderedcms', '0006_mailchimpapisettings'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='googleapisettings',
            options={'verbose_name': 'Google API'},
        ),
        migrations.AlterModelOptions(
            name='mailchimpapisettings',
            options={'verbose_name': 'Mailchimp API'},
        ),
    ]



from django.test import Client
from wagtail.tests.utils import WagtailPageTests

from coderedcms.models.page_models import (
    CoderedArticleIndexPage,
    CoderedArticlePage,
    CoderedEventIndexPage,
    CoderedEventPage,
    CoderedFormPage,
    CoderedLocationIndexPage,
    CoderedLocationPage,
    CoderedPage,
    CoderedStreamFormPage,
    CoderedWebPage,
    get_page_models
)
from coderedcms.tests.testapp.models import (
    ArticleIndexPage,
    ArticlePage,
    EventIndexPage,
    EventPage,
    FormPage,
    LocationIndexPage,
    LocationPage,
    StreamFormPage,
    WebPage
)


class BasicPageTestCase():
    """
    This is a testing mixin used to run common tests for basic versions of page types.
    """
    class Meta:
        abstract = True

    def setUp(self):
        self.client = Client()
        self.basic_page = self.model(
            title=str(self.model._meta.verbose_name)
        )
        self.homepage = WebPage.objects.get(url_path='/home/')
        self.homepage.add_child(instance=self.basic_page)

    def test_get(self):
        """
        Tests to make sure a basic version of the page serves a 200 from a GET request.
        """

        response = self.client.get(self.basic_page.url, follow=True)
        self.assertEqual(response.status_code, 200)


class AbstractPageTestCase():
    """
    This is a testing mixin used to run common tests for abstract page types.
    """
    class Meta:
        abstract = True

    def test_not_available(self):
        """
        Tests to make sure the page is not creatable and not in CodeRed CMS's global list of page models.  # noqa
        """
        self.assertFalse(self.model.is_creatable)
        self.assertFalse(self.model in get_page_models())


class ConcretePageTestCase():
    """
    This is a testing mixin used to run common tests for concrete page types.
    """
    class Meta:
        abstract = True

    def test_is_available(self):
        """
        Tests to make sure the page is creatable and in CodeRed CMS's global list of page models.
        """
        self.assertTrue(self.model.is_creatable)
        self.assertTrue(self.model in get_page_models())


class ConcreteBasicPageTestCase(ConcretePageTestCase, BasicPageTestCase):
    class Meta:
        abstract = True


class ConcreteFormPageTestCase(ConcreteBasicPageTestCase):
    class Meta:
        abstract = True

    def test_post(self):
        """
        Tests to make sure a basic version of the page serves a 200 from a POST request.
        """
        response = self.client.post(self.basic_page.url, follow=True)
        self.assertEqual(response.status_code, 200)

    def test_spam(self):
        """
        Test to check if the default spam catching works.
        """
        response = self.client.post(self.basic_page.url, {'cr-decoy-comments': 'This is Spam'}, follow=True)  # noqa
        messages = list(response.context['messages'])
        self.assertEqual(len(messages), 1)
        self.assertEqual(str(messages[0]), self.basic_page.get_spam_message())

    def test_not_spam(self):
        """
        Test to check if the default spam catching won't mark correct posts as spam.
        """
        response = self.client.post(self.basic_page.url)
        self.assertFalse(hasattr(response, 'is_spam'))


class CoderedArticleIndexPageTestCase(AbstractPageTestCase, WagtailPageTests):
    model = CoderedArticleIndexPage


class CoderedArticlePageTestCase(AbstractPageTestCase, WagtailPageTests):
    model = CoderedArticlePage


class CoderedFormPageTestCase(AbstractPageTestCase, WagtailPageTests):
    model = CoderedFormPage


class CoderedPageTestCase(WagtailPageTests):
    model = CoderedPage

    def test_not_available(self):
        self.assertFalse(self.model.is_creatable)
        self.assertTrue(self.model in get_page_models())


class CoderedWebPageTestCase(AbstractPageTestCase, WagtailPageTests):
    model = CoderedWebPage


class CoderedLocationIndexPageTestCase(AbstractPageTestCase, WagtailPageTests):
    model = CoderedLocationIndexPage


class CoderedLocationPageTestCase(AbstractPageTestCase, WagtailPageTests):
    model = CoderedLocationPage


class CoderedEventIndexPageTestCase(AbstractPageTestCase, WagtailPageTests):
    model = CoderedEventIndexPage


class CoderedEventPageTestCase(AbstractPageTestCase, WagtailPageTests):
    model = CoderedEventPage


class CoderedStreamFormPageTestCase(AbstractPageTestCase, WagtailPageTests):
    model = CoderedStreamFormPage


class ArticlePageTestCase(ConcreteBasicPageTestCase, WagtailPageTests):
    model = ArticlePage


class ArticleIndexPageTestCase(ConcreteBasicPageTestCase, WagtailPageTests):
    model = ArticleIndexPage


class FormPageTestCase(ConcreteFormPageTestCase, WagtailPageTests):
    model = FormPage


class WebPageTestCase(ConcreteBasicPageTestCase, WagtailPageTests):
    model = WebPage


class EventIndexPageTestCase(ConcreteBasicPageTestCase, WagtailPageTests):
    model = EventIndexPage


class EventPageTestCase(ConcreteBasicPageTestCase, WagtailPageTests):
    model = EventPage


class LocationIndexPageTestCase(ConcreteBasicPageTestCase, WagtailPageTests):
    model = LocationIndexPage


class LocationPageTestCase(ConcreteBasicPageTestCase, WagtailPageTests):
    model = LocationPage


class StreamFormPageTestCase(ConcreteFormPageTestCase, WagtailPageTests):
    model = StreamFormPage



from django.db import models
from django.forms.widgets import Input
from django.template import Context, Template
from django.template.loader import render_to_string
from django.utils.translation import gettext_lazy as _

from wagtail.admin.edit_handlers import FieldPanel
from wagtail.core import hooks

from coderedcms.api.mailchimp import MailchimpApi

from ast import literal_eval
import json


class MailchimpSubscriberIntegrationWidget(Input):
    template_name = 'coderedcms/formfields/mailchimp/subscriber_integration_widget.html'
    js_template_name = 'coderedcms/formfields/mailchimp/subscriber_integration_js.html'

    def get_context(self, name, value, attrs):
        ctx = super(MailchimpSubscriberIntegrationWidget, self).get_context(name, value, attrs)

        json_value = self.get_json_value(value)
        list_library = self.build_list_library()
        ctx['widget']['value'] = json.dumps(json_value)
        ctx['widget']['extra_js'] = self.render_js(name, list_library, json_value)
        ctx['widget']['selectable_mailchimp_lists'] = self.get_selectable_mailchimp_lists(
            list_library)
        ctx['widget']['stored_mailchimp_list'] = self.get_stored_mailchimp_list(json_value)

        return ctx

    def render_js(self, name, list_library, json_value):
        ctx = {
            'widget_name': name,
            'widget_js_name': name.replace('-', '_'),
            'list_library': list_library,
            'stored_mailchimp_list': self.get_stored_mailchimp_list(json_value),
            'stored_merge_fields': self.get_stored_merge_fields(json_value),
        }

        return render_to_string(self.js_template_name, ctx)

    def get_json_value(self, value):
        if value:
            json_value = json.loads(value)
        else:
            json_value = json.loads('{}')
        if 'list_id' not in json_value:
            json_value['list_id'] = ""
        if 'merge_fields' not in json_value:
            json_value['merge_fields'] = {}
        if 'email_field' not in json_value:
            json_value['email_field'] = ""
        if 'interest_categories' not in json_value:
            json_value['interest_categories'] = {}
        return json_value

    def get_stored_mailchimp_list(self, value):
        if 'list_id' in value:
            return str(value['list_id'])

    def get_stored_merge_fields(self, value):
        if 'merge_fields' in value:
            return json.dumps(value['merge_fields'])
        return json.dumps({})

    def get_selectable_mailchimp_lists(self, library):
        selectable_lists = [('', 'Please select one of your Mailchimp Lists.')]
        for k, v in library.items():
            selectable_lists.append((k, v['name']))

        return selectable_lists

    def build_list_library(self):
        mailchimp = MailchimpApi()
        list_library = {}
        if mailchimp.is_active:
            lists = mailchimp.get_lists()
            for mlist in lists['lists']:
                list_library[mlist['id']] = {
                    'name': mlist['name'],
                    'merge_fields': {},
                    'interest_categories': {}
                }

                list_library[mlist['id']]['merge_fields'] = \
                    mailchimp.get_merge_fields_for_list(mlist['id'])['merge_fields']
                list_library[mlist['id']]['interest_categories'] = \
                    mailchimp.get_interest_categories_for_list(mlist['id'])['categories']

                for category in list_library[mlist['id']]['interest_categories']:
                    category['interests'] = mailchimp.get_interests_for_interest_category(
                        mlist['id'],
                        category['id']
                    )['interests']

        return list_library


class MailchimpSubscriberIntegration(models.Model):
    class Meta:
        abstract = True

    subscriber_json_data = models.TextField(
        blank=True,
        verbose_name=_("List")
    )

    def integration_operation(self, instance, **kwargs):
        mailchimp = MailchimpApi()
        if mailchimp.is_active:
            rendered_dictionary = self.render_dictionary(
                self.format_form_submission(kwargs['form_submission']))
            mailchimp.add_user_to_list(list_id=self.get_list_id(), data=rendered_dictionary)

    def format_form_submission(self, form_submission):
        formatted_form_data = {}
        for k, v in literal_eval(form_submission.form_data).items():
            formatted_form_data[k.replace('-', '_')] = v
        return formatted_form_data

    def get_data(self):
        return json.loads(self.subscriber_json_data)

    def get_merge_fields(self):
        if 'merge_fields' in self.get_data():
            return self.get_data()['merge_fields']
        return {}

    def get_list_id(self):
        if 'list_id' in self.get_data():
            return self.get_data()['list_id']

    def combine_interest_categories(self):
        interest_dict = {}
        for category_id, value in self.get_data()['interest_categories'].items():
            interest_dict.update(value['interests'])

        return interest_dict

    def render_dictionary(self, form_submission):
        rendered_dictionary_template = json.dumps({
            'members': [
                {
                    'email_address': self.get_data()['email_field'],
                    'merge_fields': self.get_data()['merge_fields'],
                    'interests': self.combine_interest_categories(),
                    'status': 'subscribed',
                }
            ]
        })

        rendered_dictionary = Template(
            rendered_dictionary_template).render(Context(form_submission))
        return rendered_dictionary

    panels = [
        FieldPanel('subscriber_json_data', widget=MailchimpSubscriberIntegrationWidget)
    ]


@hooks.register('form_page_submit')
def run_mailchimp_subscriber_integrations(instance, **kwargs):
    if hasattr(instance, 'integration_panels'):
        for panel in instance.integration_panels:
            for integration in getattr(instance, panel.relation_name).all():
                integration.integration_operation(instance, **kwargs)
